echo Script file for generating XML Schema documentation
echo using xs3p schema documentation generator.

# For large schemas you can use 'Generate chunks' option. 
# In this case you must specify the following parameters:
# generateChunks = "yes"
# mainDocFile - The name of the generated schema documentation file, for instance personal.html
# mainDocBase - The path to the output folder where documentation will be generated, ended in file separator, for instance c:\test\


#!/bin/sh
case "$0" in
/*)
  p=$0
;;
*) p=`/bin/pwd`/$0
;;
esac
p=`dirname "$p"`

XSDPATH=$1
IMAGESFOLDER=$2
HTMLPATH=$3
DIAGRAMS_EXTENSION=$4
USE_HASH_CODES=$5

# Set default values if all parameters are missing
if [ "$XSDPATH" == "" -a "$IMAGESFOLDER" == "" -a "$HTMLPATH" == "" -a "$DIAGRAMS_EXTENSION" == "" -a "$USE_HASH_CODES" == "" ]; then 
XSDPATH="$p/samples/personal.xsd"
IMAGESFOLDER="schemaDiagrams"
HTMLPATH="$p/samples/personal.html"
DIAGRAMS_EXTENSION=".png"
USE_HASH_CODES="yes"
fi

if [ "$XSDPATH" == "" -o "$IMAGESFOLDER" == "" -o "$HTMLPATH" == "" -o "$DIAGRAMS_EXTENSION" == "" -o "$USE_HASH_CODES" == "" ]; then 
  echo Usage: schemaDocumentation pathToXSDFile imagesFolder pathToHtmlFile digramsExtesion
  echo ----------------------------------------------------------------------------------------------
  echo Parameters:
  echo 1. pathToXSDFile  - The path to XML Schema file
  echo 2. imagesFolder   - The path to the folder where generated images will be saved into. Relative 
  echo                     to the output file
  echo 3. pathToHtmlFile - The path to the html file
  echo 4. diagramsExtension - Either ".png" or ".jpg" to specify the diagrams type
  echo 5. useHashcodes - If enabled then the achors and links will be generated using the hashcode of the component
  echo      identifier instead of using the identifier itself. This is useful when the schema componet
  echo      names contain characters that are not directly supported by the browsers or by the file system. 

  echo Example: schemaDocumentation.sh samples/personal.xsd schemaDiagrams samples/personal.html .png yes
  echo ----------------------------------------------------------------------------------------------	
 else 
  echo Generate documentation for $XSDPATH
  echo Output file $HTMLPATH
  CP=$p:$p/classes:$p/lib/oxygen.jar:$p/lib/oxygen.jar:$p/lib/xml-apis.jar:$p/lib/xercesImpl.jar:$p/lib/log4j.jar:$p/lib/resolver.jar:$p/lib/jimi.zip

# generates diagrams
 java -cp "$CP" ro.sync.exml.SwingSDImageGenerator "$XSDPATH" "$IMAGESFOLDER" "$DIAGRAMS_EXTENSION" "$USE_HASH_CODES"

#generate the documentation file
  
  PARAMS="-PARAM showDiagrams yes"
  PARAMS="$PARAMS -PARAM imagesBaseURL $IMAGESFOLDER"
  PARAMS="$PARAMS -PARAM sortByComponent true"
  PARAMS="$PARAMS -PARAM useJavaScript true"
  PARAMS="$PARAMS -PARAM printAllSuperTypes true"
  PARAMS="$PARAMS -PARAM printAllSubTypes true"
  PARAMS="$PARAMS -PARAM printLegend true"
  PARAMS="$PARAMS -PARAM printGlossary true"
  PARAMS="$PARAMS -PARAM printNSPrefixes true"
  PARAMS="$PARAMS -PARAM showLogicalDiagrams true"
  PARAMS="$PARAMS -PARAM diagramImagesExtension $DIAGRAMS_EXTENSION"
  PARAMS="$PARAMS -PARAM useHashcodes $USE_HASH_CODES"
  
  java -cp "$p/lib/xalanOxygen.zip" org.apache.xalan.xslt.Process -OUT "$HTMLPATH" -IN "$XSDPATH" -XSL "$p/frameworks/xs3p/xs3p.xsl" $PARAMS
fi